// src/errors.ts
class SocketError extends Error {
  name = "SocketError";
  originalError;
  constructor(message, originalError) {
    super(message);
    this.originalError = originalError;
  }
}

// src/messages.ts
function wrap(data) {
  return ["data", data];
}
function unwrap(message) {
  return message[1];
}
function deserialize(data) {
  try {
    const parsedData = JSON.parse(data);
    if (typeof parsedData === "number") {
      return ["heartbeat", parsedData];
    }
    if (parsedData !== null && Array.isArray(parsedData) && parsedData.length === 2 && parsedData[0] === "data") {
      const data2 = parsedData[1];
      return ["data", data2];
    }
    throw new SocketError("Failed to parse internal message shape");
  } catch (error) {
    throw new SocketError("Failed to parse incoming message shape: Failed to deserialize");
  }
}

// src/socket.ts
var ONE_SECOND = 1000;
var HEARTBEAT_INTERVAL = 30 * ONE_SECOND;
var MAX_RECONNECT_DELAY = 30 * ONE_SECOND;

class ManagedSocket {
  url;
  socket;
  onMessage = () => {};
  onError = () => {};
  onConnected = () => {};
  onDisconnected = () => {};
  _reconnectAttempts = 0;
  _messageQueue = [];
  _debugMode = false;
  _offline = false;
  constructor(url, handlers = {}) {
    this.url = url;
    this.onMessage = handlers.onMessage ?? this.onMessage;
    this.onError = handlers.onError ?? this.onError;
    this.onConnected = handlers.onConnected ?? this.onConnected;
    this.onDisconnected = handlers.onDisconnected ?? this.onDisconnected;
    this.socket = this.connect();
    setInterval(() => {
      this.heartbeat();
    }, HEARTBEAT_INTERVAL);
    window.addEventListener("offline", () => {
      this._offline = true;
      if (this.isConnected) {
        this.debugMessage("Internet connection lost, closing socket...");
        this.close();
      }
    });
    window.addEventListener("online", () => {
      this._offline = false;
      if (!this.isConnected) {
        this.debugMessage("Internet connection restored, reconnecting socket...");
        this.attemptReconnection();
      }
    });
  }
  heartbeat() {
    if (this.socket?.readyState === WebSocket.OPEN) {
      this.socket.send(JSON.stringify(Date.now()));
    }
  }
  debugMessage(...args) {
    if (this._debugMode) {
      const output = ["[Socket]", ...args];
      console.log(...output);
    }
  }
  sendQueuedMessages() {
    const messagesToSend = [...this._messageQueue];
    this._messageQueue = [];
    for (const message of messagesToSend) {
      this.send(message);
    }
  }
  attemptReconnection() {
    if (this._offline) {
      this.debugMessage("Browser offline; skipping reconnection attempt");
      return;
    }
    const delay = Math.min(ONE_SECOND * 2 ** this._reconnectAttempts, MAX_RECONNECT_DELAY);
    this.debugMessage(`Attempting to reconnect in ${delay}ms`);
    setTimeout(() => {
      this._reconnectAttempts++;
      this.connect();
    }, delay);
  }
  connect() {
    const newSocket = new WebSocket(this.url);
    newSocket.addEventListener("open", () => {
      this._reconnectAttempts = 0;
      this.onConnected();
      this.sendQueuedMessages();
    });
    newSocket.addEventListener("close", ({ wasClean, code, reason }) => {
      this.debugMessage("WebSocket closed", {
        wasClean,
        code,
        reason
      });
      this.onDisconnected({
        wasClean,
        code,
        reason
      });
      if (!this._offline) {
        this.debugMessage("Attempting automatic reconnection");
        return this.attemptReconnection();
      }
    });
    newSocket.addEventListener("error", (error) => {
      this.debugMessage("WebSocket error", error);
      const socketError = new SocketError("Socket error", new Error(error.currentTarget?.toString()));
      this.onError(socketError);
    });
    newSocket.addEventListener("message", (messageEvent) => {
      const [type, data] = deserialize(messageEvent.data);
      if (type === "heartbeat") {
        return;
      }
      this.debugMessage("Received message", data);
      this.onMessage(data);
    });
    this.socket = newSocket;
    return newSocket;
  }
  get isConnected() {
    return this.socket?.readyState === WebSocket.OPEN;
  }
  debug(debugModeEnabled) {
    this._debugMode = debugModeEnabled;
  }
  close() {
    if (this.socket?.readyState !== WebSocket.OPEN) {
      throw new SocketError("Cannot close a socket that is not open");
    }
    this.socket.close();
  }
  send(data) {
    if (!this.socket || this.socket.readyState !== WebSocket.OPEN) {
      this.debugMessage("WebSocket is not open. Queuing message.");
      this._messageQueue.push(data);
      return;
    }
    this.socket.send(JSON.stringify(wrap(data)));
  }
}
export {
  wrap,
  unwrap,
  deserialize,
  SocketError,
  ManagedSocket
};
